#!/usr/bin/perl
# vim:ts=4
# winalert.pl
# Steve S 2004
#
# Reads a file to get a list of hosts/users to alert, and sends either a
# parameter message or a predefined one to them.
# winalert [-u user][-g group][-m message][-[1...9] parm[1..9]][-M messagetext][parm1 [parm2 [parm3]]]
# Parm4 defaults to the environment variable THRESH_DESC

# eg:
# winalert -g all -m msg1
# winalert -u steve -M "There is a big problem!"
# winalert -m msg2 -1 "mail system" -2 "12234 messages" 
# winalert -m msg2 "mail system" "12234 messages" 
#
# Conf file example:
#[winalert]
#smbclient = /usr/bin/smbclient
#[users]
#username = netbiosname
#steveshipway = sshi052
#ops = operator
#[groups]
# groupname = username, username, ....
#sysadmin = steveshipway
#ops = ops
#all = ops, steveshipway
#[messages]
# messagename = messagetext, possibly with %1% variables
#msg1 = There is a general problem.  Check the systems.
#msg2 = There is a problem with %1%
#msg3 = There is a problem with %1%: Monitored value at %2%
#

use strict;
use Getopt::Std;

my($CONF) = "/u01/etc/winalert.conf";

my(%messages) = ();
my(%users) = ();
my(%groups) = ();
my($SMBCLIENT) = "smbclient";
my(%params) = ();
my($MSG) = "Alert";
my(@TARGETS) = ();
use vars qw($opt_h $opt_M $opt_m $opt_u $opt_g);
use vars qw($opt_1 $opt_2 $opt_3 $opt_4 $opt_5 $opt_6 $opt_7 $opt_8 $opt_9);

sub help {
print "winalert [-u user][-g group][-m message][-[1...9] parm[1..9]][-M messagetext][ netbiosname...]\n";
print "eg:\n";
print "winalert -g all -m msg1\n";
print "winalert -u steve -M \"There is a big problem!\"\n";
print "winalert -m msg2 -1 \"mail system\" -2 \"12234 messages\" sshi052\n";
exit 0;
}

sub parseconf {
	my($line,@line);
	my($section,$key,$arg);

	open CONF, "<$CONF" or do { 
		print "Unable to open $CONF for reading!\n"; return;
	};
	while( $line = <CONF> ) {
		next if($line =~ /^\s*#/);  # skip comments
		next if($line =~ /^\s*$/);  # skip blank
		chomp $line;
		if( $line =~ /\[(.*)\]/ ) { $section = lc $1; next; }
		next if(!$section);
		$line =~ /^\s*(\S+)\s*=\s*(.*)$/;
		($key,$arg) = ((lc $1),$2);
		next if(!$key or !$arg);
		if( $section eq "winalert" ) {
			$SMBCLIENT = $arg if($key eq "smbclient");
		} elsif($section eq "users" ) {
			if(defined $users{$key}) {
				print "Error: $key redefined in [users]\n"; next; }
			$users{$key} = $arg;
		} elsif($section eq "groups" ) {
			if(defined $groups{$key}) {
				print "Error: $key redefined in [groups]\n"; next; }
			$groups{$key} = [ split /[\s,]+/,$arg ];
		} elsif($section eq "messages" ) {
			if(defined $messages{$key}) {
				print "Error: $key redefined in [messages]\n"; next; }
			$messages{$key} = $arg;
		} else { print "Warning: bad section in $CONF\n"; }
	}

	close CONF;
}

# Work out what message we will send
sub getmessage {
	my($s,$d);
	my($msg) = "There is a problem [$opt_1,$opt_2,$opt_3,$opt_4].";

	if($opt_M) { return $opt_M; }
	$opt_m = 'default' if(!$opt_m);
	if($opt_m and defined $messages{$opt_m}) { $msg = $messages{$opt_m}; }


	# paramter processing
	foreach ( qw/1 2 3 4 5 6 7 8 9/ ) {
		$s = "\%$_\%";
		eval "\$d = \$opt_$_";
		$d = "(unknown)" if(!defined $d);
		$msg =~ s/$s/$d/g;
	}
	$msg =~ s/\\n/\n/g;
	$msg =~ s/\\t/\t/g;
	return $msg;
}

# return a list of which user/host to message
sub gettargets {
	my(@t) = ();
	my($g);

	$opt_g = 'default' if(!$opt_u and !$opt_g);

	if( $opt_u ) {
		print "Notify User $opt_u\n";

		if(defined $users{$opt_u}) {
			push @t, $users{$opt_u};
		} else {
			print "Error: invalid user $opt_u\n";
		}
	}

	if( $opt_g ) {
		print "Notify Group $opt_g\n";

		if(defined $groups{$opt_g}) {
			foreach $g ( @{$groups{$opt_g}} ) {
				if(defined $users{$g}) {
					push @t, $users{$g};
				} else {
					print "Error: invalid user $g in group $opt_g\n";
				}
			}
		} else {
			print "Error: invalid group $opt_g\n";
		}
	}
	return @t;
}

sub sendto($$) {
	my($who, $what) = @_;

	return if(!$who or !$what);

	# UNIX->Windows SMB mode...
	open SMB, "|$SMBCLIENT -M $who > /dev/null " or do {
		print "Error: cannot start $SMBCLIENT\n";
		return;
	};
	print SMB "$what\n";
	close SMB;
	print "Notified $who\n";
}

##########################################################################
# MAIN CODE STARTS

parseconf;

getopts('hM:m:1:2:3:4:5:6:7:8:9:u:g:');

if( $opt_h ) { help(); exit 0; }

if( $ARGV[0] and !$opt_1 ) { $opt_1 = $ARGV[0]; }
if( $ARGV[1] and !$opt_2 ) { $opt_2 = $ARGV[1]; }
if( $ARGV[2] and !$opt_3 ) { $opt_3 = $ARGV[2]; }
if( $ENV{THRESH_DESC} and !$opt_4 ) { $opt_4 = $ENV{THRESH_DESC}; }

$MSG = getmessage;

@TARGETS = gettargets;

if(!@TARGETS) {
	print "No destination for the message!\n";
	exit 1;
}

foreach ( @TARGETS ) { sendto( $_, $MSG ); }

exit 0;
